(**********************************************************************)
(*                                                                    *)
(*                           Moca                                     *)
(*                                                                    *)
(*          Pierre Weis, INRIA Rocquencourt                           *)
(*          Frdric Blanqui, projet Protheo, INRIA Lorraine          *)
(*                                                                    *)
(*  Copyright 2005-2007,                                              *)
(*  Institut National de Recherche en Informatique et en Automatique. *)
(*  All rights reserved.                                              *)
(*                                                                    *)
(*  This file is distributed under the terms of the Q Public License. *)
(*                                                                    *)
(**********************************************************************)

(* $Id: genr_base.mli,v 1.10 2007/04/26 16:18:32 blanqui Exp $ *)

(** Base definitions for generation functions *)

open Code;;
open Spec;;

val set_sharable_target : unit -> unit;;
val get_sharable_target : unit -> bool;;
(** Do we generate maximum sharing ? *)

val genr_lhs : Parsetree.pattern -> Parsetree.pattern;;
(** Generation of a LHS. *)

val genr_rule : Parsetree.pattern -> Parsetree.expression option ->
  Parsetree.pattern -> Code.clause;;
(* Generation function for user's defined rewrite rule Rule (p1, p2). *)

(** Handling function names. *)
val construction_function_name : uident -> lident;;
(** Construction function name from constructor name. *)

val prefix_cname : string -> uident -> lident;;
(** Name generation functions from a constructor name:
   [prefix_cname p c] is the name "prefix_name",
   where "prefix" is the contents of p,
   and "name" is the lower case version of c. *)

val constr_fun1 : uident -> exp -> exp;;
val constr_fun2 : uident -> exp -> exp -> exp;;
(** Application of unary or binary construction functions associated with the
    given constructor name. *)

(** Term construction. *)

val constr : uident -> exp list -> exp;;
val constr1 : uident -> exp -> exp;;
val constr2 : uident -> exp -> exp -> exp;;

(** Pattern construction. *)

val pattern : uident -> exp list -> exp;;
val pattern1 : uident -> exp -> exp;;
val pattern2 : uident -> exp -> exp -> exp;;

val genr_eq : lident -> structure_item;;

(** Data structure for generation of construction functions. *)

type function_definition = {
  fd_uname : uident; (* uppercase name *)
  fd_relations : Parsetree.relations;
  fd_arity : int;
};;
